@@ -55,6 +55,12 @@
 /*----------------------------------------------------------------------------*/
 /**@{*/
 
+inline static int
+muse_mapidx(int i, int s, int q)
+{
+  return (kMuseNumIFUs * kMuseSlicesPerCCD * (q-1) + kMuseSlicesPerCCD * (i-1) + s - 1);
+}
+
 /* Compute the arithmetic mean */
 static void
 muse_autocalib_mean(float *data, int n, double x[3], int *indx)
@@ -319,7 +325,7 @@ muse_autocalib_applycorr(float *data, float *stat, int npix, unsigned char *ifu,
   #pragma omp parallel for default(none)                                       \
           shared(corr, data, ifu, npix, quad, sli, stat, idstring)
   for (n = 0; n < npix; n++) {
-    cpl_size k = MAPIDX(ifu[n], sli[n], quad[n]);
+    cpl_size k = muse_mapidx(ifu[n], sli[n], quad[n]);
     float c = corr[k];
     data[n] *= c;
     stat[n] *= c * c;
@@ -453,7 +459,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
       }
     }
     if (cpl_table_is_selected(aPixtable->table, n)) {
-      k = MAPIDX(ifu[n], sli[n], quad[n]);
+      k = muse_mapidx(ifu[n], sli[n], quad[n]);
       int i;
       /* atomic capture is new in OpenMP v3.1 (or _OPENMP=201107), */
 #if _OPENMP >= 201107 /* needed for atomic capture */
@@ -487,7 +493,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
           slice_count2 = 0;
       unsigned short s;
       for (s = 0; s < kMuseSlicesPerCCD; s++) {
-        k = MAPIDX(i+1, s+1, q+1);
+        k = muse_mapidx(i+1, s+1, q+1);
         int slidx = kMuseSlicesPerCCD * i + s;
         if (npts[k] > 100) {
           /* Compute the mean flux of the ifu/slice/wavelength bin,  *
@@ -621,7 +627,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
           slice_count2 = 0;
       unsigned short s;
       for (s = 0; s < kMuseSlicesPerCCD; s++) {
-        k = MAPIDX(i+1, s+1, q+1);
+        k = muse_mapidx(i+1, s+1, q+1);
         int slidx = kMuseSlicesPerCCD * i + s;
         if (npts[k] != 0) {
           if (s < kMuseSlicesPerCCD/2) {
@@ -671,7 +677,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
                       "(%.1f sigma clip)...", aClip);
       }
       for (s = 0; s < kMuseSlicesPerCCD; s++) {
-        k = MAPIDX(i+1, s+1, q+1);
+        k = muse_mapidx(i+1, s+1, q+1);
         if (s < kMuseSlicesPerCCD/2) {
           if (!slice_count1) {
             continue;
@@ -727,7 +733,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
     unsigned short s;
     for (s = 1; s <= kMuseSlicesPerCCD; s++) {
       for (q = 1; q <= nlbin; q++) {
-        k = MAPIDX(i, s, q);
+        k = muse_mapidx(i, s, q);
         if (aFactors) {
           /* record slice properties in the table */
           cpl_table_set_int((*aFactors)->table, "ifu", k, i);
@@ -752,14 +758,14 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
 
         cpl_size kprev, knext;
         if (q == 1) {
-          kprev = MAPIDX(i, s, q+1);
-          knext = MAPIDX(i, s, q+2);
+          kprev = muse_mapidx(i, s, q+1);
+          knext = muse_mapidx(i, s, q+2);
         } else if (q == nlbin) {
-          kprev = MAPIDX(i, s, q-1);
-          knext = MAPIDX(i, s, q-2);
+          kprev = muse_mapidx(i, s, q-1);
+          knext = muse_mapidx(i, s, q-2);
         } else {
-          kprev = MAPIDX(i, s, q-1);
-          knext = MAPIDX(i, s, q+1);
+          kprev = muse_mapidx(i, s, q-1);
+          knext = muse_mapidx(i, s, q+1);
         }
         meanq = (corr[kprev] + corr[knext]) / 2.;
         threshq = CPL_MAX(0.03, 3 * fabsf(corr[kprev] - corr[knext]));
