@@ -224,7 +224,7 @@ muse_lsf_select_arc_line(muse_pixtable *aPixtable, double aLambda,
 
 /*----------------------------------------------------------------------------*/
 /**
-   @brief Read images and combine the arc lines into one pixtable.
+   @brief Read images and combine the arc lines into one pixel table.
    @param aImages     list of arc images
    @param aTrace      table containing the trace solution
    @param aWave       table containing the wavelength calibration solution
@@ -238,7 +238,7 @@ muse_lsf_select_arc_line(muse_pixtable *aPixtable, double aLambda,
    @cpl_ensure{aArcLines, CPL_ERROR_NULL_INPUT, NULL}
 
    Create pixel tables from all images in the aImages list, select the pixels
-   around each arc line, and store the resulting pixels in a new pixtable.
+   around each arc line, and store the resulting pixels in a new pixel table.
 
    The output pixel table will have additional columns
    <tt>line_lambda</tt>, <tt>line_flux</tt>, and
@@ -341,7 +341,7 @@ muse_lsf_create_arcpixtable(muse_imagelist *aImages,
 /*----------------------------------------------------------------------------*/
 /**
    @private
-   @brief Do a polynomial fit on an arc pixtable
+   @brief Do a polynomial fit on an arc pixel table.
    @param aPixtable      special pixel table with arc lines
    @param aOrderLsf      polynomial order in x (LSF) direction
    @param aOrderLambda   polynomial order in y (arc line wavelength) direction
@@ -461,7 +461,7 @@ muse_lsf_fit_slice(const muse_pixtable *aPixtable, cpl_image *aLsfImage,
                " with %"CPL_SIZE_FORMAT" entries",
                i_ifu, i_slice, nrows);
 
-  /* Create temporary column with (lmbda - line_lambda) and sort pixtable
+  /* Create temporary column with (lmbda - line_lambda) and sort pixel table
      according to this column. Sorting is important since we use it later
      to extract the values according to the regession window using
      find_sorted(). */
@@ -769,12 +769,12 @@ muse_lsf_cube_load(const char *aFileName, unsigned char aIFU)
 /**
    @brief Create an average image from all LSF cubes
    @param aLsfCube    array of all LSF cubes
-   @param aPixtable   pixtable for the weighting, or NULL
+   @param aPixtable   pixel table for the weighting, or NULL
    @return Average LSF image
 
    All images from all cubes are squeezed into one image. If a
-   pixtable is given, each IFU/slice is weighted with the number of
-   entries in the pixtable for this IFU/slice.
+   pixel table is given, each IFU/slice is weighted with the number of
+   entries in the pixel table for this IFU/slice.
  */
 /*----------------------------------------------------------------------------*/
 cpl_image *
@@ -956,7 +956,7 @@ muse_lsf_apply(const cpl_image *aLsfImage, const muse_wcs *aWCS,
     cpl_array_set(lsf_lambda, i+1, aWCS->crval1 + (i - aWCS->crpix1) * aWCS->cd11);
   }
   // Set outer limits of LSF to zero to avoid invalid values in the fit
-  // +- 10000 A is used since the pixtable range will never be in this range
+  // +- 10000 A is used since the pixel table range will never be in this range
   cpl_array_set(lsf_lambda, 0, -10000);
   cpl_array_set(lsf_values, 0, 0);
   cpl_array_set(lsf_lambda, 1, aWCS->crval1 - aWCS->crpix1 * aWCS->cd11);
@@ -992,7 +992,7 @@ muse_lsf_apply(const cpl_image *aLsfImage, const muse_wcs *aWCS,
 
 /*----------------------------------------------------------------------------*/
 /**
-   @brief  Add line spread function to a MUSE slice of a pixtable
+   @brief  Add line spread function to a MUSE slice of a pixel table.
    @param  aPixtable   table containing one slice, with wavelength and
                        an lsf column
    @param  aWavelength line wave length
@@ -1000,10 +1000,9 @@ muse_lsf_apply(const cpl_image *aLsfImage, const muse_wcs *aWCS,
    @param  aLsfImage   image containing the line spread function for the slice
    @param  aLsfWCS     WCS definition for aLsfImage
 
-   This will add the LSF for one line to the pixtable. The pixtable is
-   required to hold an "lsf" column with values set to
-   zero. Subsequent calls of this function add the line spread
-   functions of each call.
+   This will add the LSF for one line to the pixel table. The pixel table is
+   required to hold an "lsf" column with values set to zero. Subsequent calls of
+   this function add the line spread functions of each call.
 */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
@@ -1029,8 +1028,9 @@ muse_lsf_apply_slice(muse_pixtable *aPixtable, double aWavelength,
   }
 
   double l_min = aLsfWCS->crval1 + aLsfWCS->cd11 * (1 - aLsfWCS->crpix1),
-    l_max = aLsfWCS->crval1
-    + aLsfWCS->cd11 * (cpl_image_get_size_x(aLsfImage) - aLsfWCS->crpix1);
+         l_max = aLsfWCS->crval1
+               + aLsfWCS->cd11 * (cpl_image_get_size_x(aLsfImage)
+                                  - aLsfWCS->crpix1);
   cpl_size imin = muse_cplarray_find_sorted(lambda, aWavelength + l_min);
   cpl_size imax = muse_cplarray_find_sorted(lambda, aWavelength + l_max);
   cpl_array *l0 = cpl_array_extract(lambda, imin, imax-imin+1);
