@@ -53,6 +53,13 @@ const char *muse_pfits_get_progid(const cpl_propertylist *);
 const char *muse_pfits_get_targname(const cpl_propertylist *);
 long muse_pfits_get_obsid(const cpl_propertylist *);
 
+/* AO/GALACSI properties */
+cpl_boolean muse_pfits_get_ho_loop(const cpl_propertylist *);
+cpl_boolean muse_pfits_get_tt_loop(const cpl_propertylist *);
+cpl_boolean muse_pfits_get_ir_loop(const cpl_propertylist *);
+const char *muse_pfits_get_ao_mirror_name(const cpl_propertylist *, unsigned int);
+const char *muse_pfits_get_ao_opti_name(const cpl_propertylist *, unsigned int);
+
 /* instrument properties */
 muse_ins_mode muse_pfits_get_mode(const cpl_propertylist *);
 const char *muse_pfits_get_insmode(const cpl_propertylist *);
@@ -136,6 +143,7 @@ const char *muse_pfits_get_pam2_filter(const cpl_propertylist *);
 int muse_pfits_get_lampnum(const cpl_propertylist *);
 const char *muse_pfits_get_lamp_name(const cpl_propertylist *, int);
 int muse_pfits_get_lamp_status(const cpl_propertylist *, int);
+double muse_pfits_get_lamp_ontime(const cpl_propertylist *, int);
 const char *muse_pfits_get_shut_name(const cpl_propertylist *, int);
 int muse_pfits_get_shut_status(const cpl_propertylist *, int);
 
