@@ -303,6 +303,9 @@ muse_postproc_get_weight_type(const char *aWeightString)
   if (!strncmp(aWeightString, "fwhm", 5)) {
     return MUSE_XCOMBINE_FWHM;
   }
+  if (!strncmp(aWeightString, "header", 7)) {
+    return MUSE_XCOMBINE_HEADER;
+  }
   if (!strncmp(aWeightString, "none", 5)) {
     return MUSE_XCOMBINE_NONE;
   }
@@ -825,7 +828,7 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
     muse_mask_delete(sky_maskr[i]);
   } /* for i (both Raman ranges) */
 
-  /* Add the LSF column to each pixtable, and do the fit. */
+  /* Add the LSF column to each pixel table, and do the fit. */
   for (i = 0; i < MUSE_RAMAN_RANGES; i++) {
     if (!pt[i]) {
       continue;
@@ -1436,6 +1439,15 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
                       cpl_error_get_message());
       cpl_errorstate_dump(prestate, CPL_FALSE, muse_cplerrorstate_dump_some);
     }
+    /* copy solution into the detection image */
+    if (wcs->wcs && wcs->cube && wcs->cube->recimages) {
+      /* image 1 is the detection image, see muse_wcs_locate_sources() */
+      muse_image *mimage = muse_imagelist_get(wcs->cube->recimages, 1);
+      if (mimage) {
+        muse_wcs_copy_keywords(wcs->wcs, mimage->header, 'S', "Solution");
+      }
+    } /* if WCS and cube */
+
     return wcs;
   } /* if MUSE_POSTPROC_ASTROMETRY */
 
