@@ -1419,7 +1419,6 @@ muse_trace_table_get_polys_for_slice(const cpl_table *aTable,
 
   cpl_polynomial **ptrace = cpl_calloc(MUSE_TRACE_NPOLY,
                                        sizeof(cpl_polynomial *));
-  char colname[7]; /* "tcI_NN" plus null byte */
   int ipoly;
   for (ipoly = 0; ipoly < MUSE_TRACE_NPOLY; ipoly++) {
     int traceorder = muse_trace_table_get_order(aTable);
@@ -1429,7 +1428,7 @@ muse_trace_table_get_polys_for_slice(const cpl_table *aTable,
     int k;
     for (k = 0; k <= traceorder; k++) {
       cpl_size pows[1] = { k }; /* trick to access the polynomial */
-      sprintf(colname, MUSE_TRACE_TABLE_COL_COEFF, ipoly, k);
+      char *colname = cpl_sprintf(MUSE_TRACE_TABLE_COL_COEFF, ipoly, k);
       int err;
       cpl_polynomial_set_coeff(ptrace[ipoly], pows,
                                cpl_table_get(aTable, colname, irow, &err));
@@ -1441,8 +1440,10 @@ muse_trace_table_get_polys_for_slice(const cpl_table *aTable,
         cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_OUTPUT, "Trace table "
                               "broken in slice %hu (row index %d) column %s",
                               aSlice, irow, colname);
+        cpl_free(colname);
         return NULL;
       } /* if */
+      cpl_free(colname);
     } /* for k */
   } /* for ipoly */
 
