@@ -40,6 +40,7 @@
 #include "muse_resampling.h"
 #include "muse_utils.h"
 
+#include "muse_kdmatch.h"
 /*----------------------------------------------------------------------------*
  *                             Debugging Macros                               *
  *         Set these to 1 or higher for (lots of) debugging output            *
@@ -633,8 +634,11 @@ muse_wcs_locate_sources(muse_pixtable *aPixtable, float aSigma,
   @param   aWCSObj      the MUSE WCS object
   @param   aReference   reference coordinates for the same field
   @param   aRadius      initial radius for pattern matching in pixels
-  @param   aFAccuracy   factor to set initial accuracy relative to mean position
-                        accuracy in aWCSObj->detected
+  @param   aFAccuracy   If a positive number, factor to set initial accuracy
+                        relative to mean position accuracy in
+                        aWCSObj->detected in the "triangle" pattern matching
+                        algorith. A value of zero switches to the kdtree based
+                        quadruple pattern matching algorithm.
   @param   aIter        number of iterations
   @param   aThresh      the rejection threshold
   @return  CPL_ERROR_NONE on success, another CPL error code on failure
@@ -644,15 +648,31 @@ muse_wcs_locate_sources(muse_pixtable *aPixtable, float aSigma,
            modified in that they are sorted by increasing source flux.
 
   Identify reference objects for the detected objects using pattern matching.
-  Start using a search radius of aRadius pixels, and iteratively decrease it,
-  until no duplicate detections are identified any more. Similarly, iterate the
-  data accuracy (decrease it downwards from the mean positioning error
-  aFAccuracy) until matches are found. Remove the remaining unidentified
-  objects.
 
-  The final values used for relative and absolute accuracy and the radius are
-  recorded in the header keywords MUSE_HDR_WCS_RADIUS, MUSE_HDR_WCS_ACCURACY,
-  and MUSE_HDR_WCS_FACCURACY.
+  There are two pattern matching algorithm implemented, which can be
+  selected by chosing a positive or zero/negative value of aFAccuracy.
+
+  In the first method (with a positive value of aFAccuracy), start
+  using a search radius of aRadius pixels, and iteratively decrease
+  it, until no duplicate detections are identified any
+  more. Similarly, iterate the data accuracy (decrease it downwards
+  from the mean positioning error aFAccuracy) until matches are
+  found. Remove the remaining unidentified objects.
+
+  The second method (when aFAccuracy is set to zero), iterates through
+  all quadruples in both the detected objects and the catalogue,
+  calculates the transformation and checks whether more than 80% of
+  the detections match a catalog entry within aRadius.
+
+  The final values used for relative and absolute accuracy and the
+  radius are recorded in the header keywords MUSE_HDR_WCS_RADIUS,
+  MUSE_HDR_WCS_ACCURACY (not when using quadruples), and
+  MUSE_HDR_WCS_FACCURACY (not when using quadruples).
+
+  The sky coordinates that were associated to each detection are recorded in
+  the (new) columns RA and DEC of the table of detected objects
+  (aWCSObj->detected). If no reference position could be identified for a given
+  detection, the RA and DEC entries will remain invalid.
 
   Derive the world coordinate plate solution using cpl_wcs_platesol() (taking
   into account shift, 2 scales, and rotation) and add it as cpl_propertylist
@@ -677,13 +697,14 @@ muse_wcs_solve(muse_wcs_object *aWCSObj, cpl_table *aReference,
   cpl_table *detected = aWCSObj->detected;
   int ndet = cpl_table_get_nrow(detected),
       nref = cpl_table_get_nrow(aReference);
+  cpl_boolean use_triangle_ppm = (aFAccuracy > 0);
   cpl_ensure_code(ndet > 0 && nref > 0, CPL_ERROR_ILLEGAL_INPUT);
   cpl_ensure_code(muse_cpltable_check(detected, muse_wcs_detections_def)
                   == CPL_ERROR_NONE &&
                   muse_cpltable_check(aReference, muse_wcs_reference_def)
                   == CPL_ERROR_NONE,
                   CPL_ERROR_BAD_FILE_FORMAT);
-  cpl_ensure_code(aRadius > 0.&& aFAccuracy > 0., CPL_ERROR_ILLEGAL_INPUT);
+  cpl_ensure_code(aRadius > 0. && aFAccuracy >= 0., CPL_ERROR_ILLEGAL_INPUT);
 
   /* sort tables by the source brightness */
   cpl_propertylist *order = cpl_propertylist_new();
@@ -761,69 +782,88 @@ muse_wcs_solve(muse_wcs_object *aWCSObj, cpl_table *aReference,
          radius = aRadius; /* start with (hopefully) large search radius */
   int nid = INT_MAX; /* number of identified detections */
   cpl_array *aid = NULL;
-  cpl_boolean dupes = CPL_FALSE;
   double xscale, yscale;
-  do {
-    double scale, angle;
-    /* As recommended in the CPL docs, initially select the 20 first (brightest!) *
-     * detections and the 10 first (brightest) reference sources, so that         *
-     * hopefully all reference sources are contained in the brightest detections. */
+  muse_wcs_get_scales(wcsin, &xscale, &yscale);
+  if (use_triangle_ppm) {
+    cpl_boolean dupes = CPL_FALSE;
+    do {
+      double scale, angle;
+      /* As recommended in the CPL docs, initially select the 20 first (brightest!) *
+       * detections and the 10 first (brightest) reference sources, so that         *
+       * hopefully all reference sources are contained in the brightest detections. */
 #define USE_DATA 15
 #define USE_PATT 10
-    int ndata = ndet < USE_DATA ? ndet : USE_DATA,
+      int ndata = ndet < USE_DATA ? ndet : USE_DATA,
         npatt = nref < USE_PATT ? nref : USE_PATT;
-    cpl_array_delete(aid);
-    do {
-      cpl_msg_debug(__func__, "trying pattern matching with accuracy %g and "
-                    "radius %g", accuracy, radius);
-      aid = cpl_ppm_match_points(data, ndata, accuracy,
-                                 patt, npatt, 1. /* [pix] */,
-                                 0.1 /* 10% */, radius, NULL, NULL,
-                                 &scale, &angle);
-      /* decrease accuracy in case pattern matching didn't succeed */
-      accuracy /= aid ? 1. : 1.5;
-      if (accuracy < FLT_EPSILON) {
-        break; /* doesn't make sense to go any lower */
-      }
-    } while (!aid); /* no matched points likely means to low accuracy */
-    cpl_errorstate state = cpl_errorstate_get();
-    nid = cpl_array_get_size(aid);
-    if (nid > 0) { /* subtract valid only if the array exists */
-      nid -= cpl_array_count_invalid(aid);
-    }
-#if 0
-    printf("aid (valid=%d):\n", nid);
-    cpl_array_dump(aid, 0, cpl_array_get_size(aid), stdout);
-    fflush(stdout);
-#endif
-    if (nid < 1) {
       cpl_array_delete(aid);
-      cpl_matrix_delete(data);
-      cpl_matrix_delete(patt);
-      cpl_errorstate_set(state); /* swallow error about NULL cpl_array */
-      cpl_propertylist_delete(wcsin);
-      return cpl_error_set_message(__func__, CPL_ERROR_DATA_NOT_FOUND, "None of "
-                                   "the %d detections could be identified with "
-                                   "the %d reference positions with radius %.3f "
-                                   "pix (starting value %.3f) and data accuracy "
-                                   "%.3e pix (intrinsic mean error %.3e)", ndet,
-                                   nref, radius, aRadius, accuracy, accuracy0);
-    }
-    /* the first-guess scale computed here only makes sense    *
-     * if multiplied by the pixel scale in the first-guess WCS */
-    muse_wcs_get_scales(wcsin, &xscale, &yscale);
-    dupes = muse_cplarray_has_duplicate(aid);
-    cpl_msg_debug(__func__, "%d %sidentified points [scale = %g, angle = %g; "
-                  "used radius = %.3f pix (starting value %.3f), data accuracy "
-                  "= %.3e pix (intrinsic mean error %.3e)]", nid,
-                  dupes ? "(non-unique!) " : "unique ",
-                  scale*1800.*(xscale+yscale), angle, radius, aRadius, accuracy,
-                  accuracy0);
-    radius /= 1.25; /* next loop with smaller radius, if necessary */
-  } while (dupes);
+      do {
+        cpl_msg_debug(__func__, "trying pattern matching with accuracy %g and "
+                      "radius %g", accuracy, radius);
+        aid = cpl_ppm_match_points(data, ndata, accuracy,
+                                   patt, npatt, 1. /* [pix] */,
+                                   0.1 /* 10% */, radius, NULL, NULL,
+                                   &scale, &angle);
+        /* decrease accuracy in case pattern matching didn't succeed */
+        accuracy /= aid ? 1. : 1.5;
+        if (accuracy < FLT_EPSILON) {
+          break; /* doesn't make sense to go any lower */
+        }
+      } while (!aid); /* no matched points likely means to low accuracy */
+      cpl_errorstate state = cpl_errorstate_get();
+      nid = cpl_array_get_size(aid);
+      if (nid > 0) { /* subtract valid only if the array exists */
+        nid -= cpl_array_count_invalid(aid);
+      }
+#if 0
+      printf("aid (valid=%d):\n", nid);
+      cpl_array_dump(aid, 0, cpl_array_get_size(aid), stdout);
+      fflush(stdout);
+#endif
+      if (nid < 1) {
+        cpl_array_delete(aid);
+        cpl_matrix_delete(data);
+        cpl_matrix_delete(patt);
+        cpl_errorstate_set(state); /* swallow error about NULL cpl_array */
+        cpl_propertylist_delete(wcsin);
+        return cpl_error_set_message(__func__, CPL_ERROR_DATA_NOT_FOUND, "None of "
+                                     "the %d detections could be identified with "
+                                     "the %d reference positions with radius %.3f "
+                                     "pix (starting value %.3f) and data accuracy "
+                                     "%.3e pix (intrinsic mean error %.3e)", ndet,
+                                     nref, radius, aRadius, accuracy, accuracy0);
+      }
+      /* the first-guess scale computed here only makes sense    *
+       * if multiplied by the pixel scale in the first-guess WCS */
+      dupes = muse_cplarray_has_duplicate(aid);
+      cpl_msg_debug(__func__, "%d %sidentified points [scale = %g, angle = %g; "
+                    "used radius = %.3f pix (starting value %.3f), data accuracy "
+                    "= %.3e pix (intrinsic mean error %.3e)]", nid,
+                    dupes ? "(non-unique!) " : "unique ",
+                    scale*1800.*(xscale+yscale), angle, radius, aRadius, accuracy,
+                    accuracy0);
+      radius /= 1.25; /* next loop with smaller radius, if necessary */
+    } while (dupes);
+  } else {
+    int n_matches = 0.8 * ndet;
+    double dist_cut = 1e-3;      /* cut on accuracy of quad area ratios */
+    cpl_msg_debug(__func__, "calling muse_ppm_kdmatch_point()");
+    aid = muse_ppm_kdmatch_points(data, 120, patt, 150, dist_cut, aRadius, n_matches);
+    nid = cpl_array_get_size(aid) - cpl_array_count_invalid(aid);
+  }
   cpl_matrix_delete(data);
   cpl_matrix_delete(patt);
 
+  /* add new columns for associated sky coordinates, erase *
+   * pre-existing ones of the same name, if they exist     */
+  if (cpl_table_has_column(detected, "RA")) {
+    cpl_table_erase_column(detected, "RA");
+  }
+  cpl_table_new_column(detected, "RA", CPL_TYPE_DOUBLE);
+  if (cpl_table_has_column(detected, "DEC")) {
+    cpl_table_erase_column(detected, "DEC");
+  }
+  cpl_table_new_column(detected, "DEC", CPL_TYPE_DOUBLE);
+
   /* create matrices again for cpl_wcs_platesol(), this    *
    * time with detected pixel positions and sky positions, *
    * but only for the identified detections                */
@@ -835,17 +875,19 @@ muse_wcs_solve(muse_wcs_object *aWCSObj, cpl_table *aReference,
       continue;
     }
     int idata = cpl_array_get_int(aid, i, NULL); /* index in detections table */
+    double ra = cpl_table_get(aReference, "RA", i, NULL),
+           dec = cpl_table_get(aReference, "DEC", i, NULL);
     cpl_matrix_set(mpx, iid, 0, cpl_table_get(detected, "XPOS", idata, NULL));
     cpl_matrix_set(mpx, iid, 1, cpl_table_get(detected, "YPOS", idata, NULL));
-    cpl_matrix_set(msky, iid, 0, cpl_table_get(aReference, "RA", i, NULL));
-    cpl_matrix_set(msky, iid, 1, cpl_table_get(aReference, "DEC", i, NULL));
+    cpl_matrix_set(msky, iid, 0, ra);
+    cpl_matrix_set(msky, iid, 1, dec);
 #if 0
-    printf("matrices: %2d\t%.7f %.7f\t%6.2f %6.2f\n", iid + 1,
-           cpl_table_get(aReference, "RA", i, NULL),
-           cpl_table_get(aReference, "DEC", i, NULL),
+    printf("matrices: %2d\t%.7f %.7f\t%6.2f %6.2f\n", iid + 1, ra, dec,
            cpl_table_get(detected, "XPOS", idata, NULL),
            cpl_table_get(detected, "YPOS", idata, NULL));
 #endif
+    cpl_table_set(detected, "RA", idata, ra);
+    cpl_table_set(detected, "DEC", idata, dec);
     iid++;
   }
 #if 0
@@ -919,13 +961,15 @@ muse_wcs_solve(muse_wcs_object *aWCSObj, cpl_table *aReference,
   cpl_propertylist_update_float(wcsout, MUSE_HDR_WCS_RADIUS, radius);
   cpl_propertylist_set_comment(wcsout, MUSE_HDR_WCS_RADIUS,
                                MUSE_HDR_WCS_RADIUS_C);
-  cpl_propertylist_update_float(wcsout, MUSE_HDR_WCS_ACCURACY, accuracy);
-  cpl_propertylist_set_comment(wcsout, MUSE_HDR_WCS_ACCURACY,
-                               MUSE_HDR_WCS_ACCURACY_C);
-  cpl_propertylist_update_float(wcsout, MUSE_HDR_WCS_FACCURACY,
-                                accuracy / accuracy0);
-  cpl_propertylist_set_comment(wcsout, MUSE_HDR_WCS_FACCURACY,
-                               MUSE_HDR_WCS_FACCURACY_C);
+  if (accuracy > 0) {
+    cpl_propertylist_update_float(wcsout, MUSE_HDR_WCS_ACCURACY, accuracy);
+    cpl_propertylist_set_comment(wcsout, MUSE_HDR_WCS_ACCURACY,
+                                 MUSE_HDR_WCS_ACCURACY_C);
+    cpl_propertylist_update_float(wcsout, MUSE_HDR_WCS_FACCURACY,
+                                  accuracy / accuracy0);
+    cpl_propertylist_set_comment(wcsout, MUSE_HDR_WCS_FACCURACY,
+                                 MUSE_HDR_WCS_FACCURACY_C);
+  }
 
   aWCSObj->wcs = wcsout;
   return rc;
@@ -1014,7 +1058,7 @@ muse_wcs_optimize_solution(muse_wcs_object *aWCSObj, float aDetSigma,
   cpl_ensure_code(nref > 0, CPL_ERROR_ILLEGAL_INPUT);
   cpl_ensure_code(muse_cpltable_check(aReference, muse_wcs_reference_def)
                   == CPL_ERROR_NONE, CPL_ERROR_BAD_FILE_FORMAT);
-  cpl_ensure_code(aRadius > 0.&& aFAccuracy > 0., CPL_ERROR_ILLEGAL_INPUT);
+  cpl_ensure_code(aRadius > 0.&& aFAccuracy >= 0., CPL_ERROR_ILLEGAL_INPUT);
 
   float detsigma = fabsf(aDetSigma),
         lolimit = 0.9999; /* 1.0 with a bit of margin */
@@ -1931,8 +1975,9 @@ muse_wcs_new(cpl_propertylist *aHeader)
   if (wcs->cddet == 0.) {
     cpl_error_set(__func__, CPL_ERROR_SINGULAR_MATRIX);
   }
-
   /* wcs->iscelsph defaults to 0 = CPL_FALSE, leave it at that */
+
+#if 0
   if (getenv("MUSE_DEBUG_WCS") && atoi(getenv("MUSE_DEBUG_WCS")) > 0) {
     cpl_msg_debug(__func__, "wcs: axis1 = { %f %f %e }, axis2 = { %f %f %e }, "
                   "crossterms = { %e %e }, det = %e",
@@ -1940,7 +1985,78 @@ muse_wcs_new(cpl_propertylist *aHeader)
                   wcs->crpix2, wcs->crval2, wcs->cd22,
                   wcs->cd12, wcs->cd21, wcs->cddet);
   }
+#endif
   return wcs;
 } /* muse_wcs_new() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief   Append WCS keywords to another header using a postfix.
+  @param   aWCS       the header containing the WCS to be copied
+  @param   aHeader    the header into which the keywords are copied
+  @param   aPostfix   the postfix (space or 'A' to 'Z')
+  @param   aWCSName   the WCSNAMEa keyword to set
+  @return  CPL_ERROR_NONE for success, any other value for failure
+
+  This function copies the existing WCS keywords, defined as in the regular
+  expression @c CPL_WCS_REGEXP, from the aWCS to aHeader, using the given
+  aPostfix to append the keyword postfix (see section "Alternative WCS Axis
+  Descriptions" in the FITS Standard v3.0).
+
+  This function only appends the new header entries, it does not check for
+  duplication. So the caller needs to make sure that none of the appended
+  keywords is a duplicate of an existing one.
+  The caller also needs to make sure that the world coordinates in aWCS
+  are appropriate for the destination header, i.e. that not a 2D WCS is copied
+  into a header for a 3D object or vice versa. Finally, the caller should make
+  sure that no other secondary WCS is present in aWCS, otherwise this gets
+  copied (and mangled) as well.
+
+  @error{set and return CPL_ERROR_NULL_INPUT, aWCS and/or aHeader is NULL}
+  @error{propagate error code of cpl_propertylist_append(),
+         copying the keywords fails}
+ */
+/*----------------------------------------------------------------------------*/
+cpl_error_code
+muse_wcs_copy_keywords(const cpl_propertylist *aWCS, cpl_propertylist *aHeader,
+                       char aPostfix, const char *aWCSName)
+{
+  cpl_ensure_code(aWCS && aHeader, CPL_ERROR_NULL_INPUT);
+
+  /* copy WCS keywords from input header */
+  cpl_propertylist *hwcs = cpl_propertylist_new();
+  cpl_propertylist_copy_property_regexp(hwcs, aWCS, CPL_WCS_REGEXP, 0);
+  /* add name to the new WCS, if one was given */
+  if (aWCSName) {
+    cpl_propertylist_update_string(hwcs, "WCSNAME", aWCSName);
+  }
+
+  /* rename all properties to include the given postfix */
+  int ip, np = cpl_propertylist_get_size(hwcs);
+  for (ip = 0; ip < np; ip++) {
+    cpl_property *p = cpl_propertylist_get(hwcs, ip);
+    char *newname = cpl_sprintf("%s%c", cpl_property_get_name(p), aPostfix);
+#if 0
+    cpl_msg_debug(__func__, "%s -> %s", cpl_property_get_name(p), newname);
+#endif
+    cpl_property_set_name(p, newname);
+    cpl_free(newname);
+  } /* for ip (all properties) */
+
+  cpl_error_code rc = cpl_propertylist_append(aHeader, hwcs);
+  cpl_errorstate state = cpl_errorstate_get();
+  const char *name = muse_pfits_get_extname(aHeader);
+  if (name) {
+    cpl_msg_debug(__func__, "Appended WCS keywords to extension %s, using "
+                  "keyword postfix %c", name, aPostfix);
+  } else {
+    cpl_errorstate_set(state);
+    cpl_msg_debug(__func__, "Appended WCS keywords to unnamed extension, using "
+                  "keyword postfix %c", aPostfix);
+  }
+  cpl_propertylist_delete(hwcs);
+
+  return rc;
+} /* muse_wcs_copy_keywords() */
+
 /**@}*/
