@@ -178,12 +178,14 @@ muse_xcombine_weights_infotable(muse_pixtable **aPixtables)
   @remark This function adds a FITS header (@ref MUSE_HDR_PT_WEIGHTED) with the
           boolean value 'T' to the pixel table, for information.
 
-  For exposure time weighting (aWeighting == MUSE_XCOMBINE_EXPTIME), compute
-  the ratio of the exposure time of the current exposure with the one from the
-  first exposure, and store this ratio as weight in a new table column. For
-  FWHM-based weighting (aWeighting == MUSE_XCOMBINE_FWHM), use the DIMM seeing
-  information, and multiply the FWHM ratio by the exposure-weight to form the
-  final weight.
+  For exposure time weighting (aWeighting == MUSE_XCOMBINE_EXPTIME), compute the
+  ratio of the exposure time of the current exposure with the one from the first
+  exposure, and store this ratio as weight in a new table column. For FWHM-based
+  weighting (aWeighting == MUSE_XCOMBINE_FWHM), use the DIMM seeing information,
+  and multiply the FWHM ratio by the exposure-weight to form the final weight.
+  For header-based weighting (aWeighting == MUSE_XCOMBINE_HEADER), multiply
+  instead the factor from the header (ESO.DRS.MUSE.WEIGHT) to the
+  exposure-weight.
 
   If aWeighting is MUSE_XCOMBINE_NONE, this function only checks for valid
   inputs, but then returns without doing anything.
@@ -199,6 +201,8 @@ muse_xcombine_weights_infotable(muse_pixtable **aPixtables)
          the exposure time header is missing from one or more input pixel tables}
   @error{output warning and set CPL_ERROR_DATA_NOT_FOUND\, switch to EXPTIME-weighting\, but return CPL_ERROR_NONE,
          all FWHM header entries are missing from one or more input pixel tables although MUSE_XCOMBINE_FWHM is requrested}
+  @error{output warning and propagate error state\, CPL_ERROR_DATA_NOT_FOUND\, but return CPL_ERROR_NONE,
+         MUSE_XCOMBINE_HEADER is requested\, but the header is missing from one or more exposures}
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
@@ -212,7 +216,8 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
     cpl_msg_info(__func__, "%d tables, not weighting them", npt);
     return CPL_ERROR_NONE;
   }
-  if (aWeighting != MUSE_XCOMBINE_EXPTIME && aWeighting != MUSE_XCOMBINE_FWHM) {
+  if (aWeighting != MUSE_XCOMBINE_EXPTIME && aWeighting != MUSE_XCOMBINE_FWHM &&
+      aWeighting != MUSE_XCOMBINE_HEADER) {
     cpl_msg_warning(__func__, "Unknown exposure weighting scheme (%d)",
                     aWeighting);
     return cpl_error_set(__func__, CPL_ERROR_UNSUPPORTED_MODE);
@@ -263,6 +268,8 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
     } else {
       fwhm0 = cpl_table_get_double(tinfo, fwhmcolumn, 0, &err);
     }
+  } else if (aWeighting == MUSE_XCOMBINE_HEADER) {
+    cpl_msg_info(__func__, "%d tables to be weighted using HEADER & EXPTIME.", npt);
   } else {
     /* simple exptime-time weighting */
     cpl_msg_info(__func__, "%d tables to be weighted using EXPTIME.", npt);
@@ -277,6 +284,20 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
       cpl_table_new_column(aPixtables[i]->table, MUSE_PIXTABLE_WEIGHT,
                            CPL_TYPE_FLOAT);
     }
+    char *headstr = NULL;
+    if (aWeighting == MUSE_XCOMBINE_HEADER) {
+      cpl_errorstate state = cpl_errorstate_get();
+      const char *key = "ESO DRS MUSE WEIGHT";
+      double w = cpl_propertylist_get_double(aPixtables[i]->header, key);
+      if (!cpl_errorstate_is_equal(state)) {
+        cpl_msg_warning(__func__, "weight header (%s) does not exist!", key);
+      } else if (w <= 0.) {
+        cpl_msg_warning(__func__, "weight from header is non-positive (%e)!", w);
+      } else {
+        headstr = cpl_sprintf(", HEADER = %.3f", w);
+        weight *= w;
+      }
+    }
     /* modify the "weight" column depending on ambient seeing */
     char *fwhmstr = NULL;
     if (fwhmcolumn) {
@@ -290,9 +311,10 @@ muse_xcombine_weights(muse_pixtable **aPixtables, muse_xcombine_types aWeighting
         fwhmstr = cpl_sprintf(", FWHM = %.2f", fwhm);
       }
     }
-    cpl_msg_debug(__func__, "Table %d, weight = %f (EXPTIME = %f%s)", i+1, weight,
-                  exptime, fwhmstr ? fwhmstr : "");
+    cpl_msg_debug(__func__, "Table %d, weight = %f (EXPTIME = %f%s%s)", i+1, weight,
+                  exptime, fwhmstr ? fwhmstr : "", headstr ? headstr : "");
     cpl_free(fwhmstr);
+    cpl_free(headstr);
     cpl_table_fill_column_window_float(aPixtables[i]->table,
                                        MUSE_PIXTABLE_WEIGHT,
                                        0, muse_pixtable_get_nrow(aPixtables[i]),
