@@ -32,6 +32,11 @@
 #include <muse.h>
 #include "muse_lingain_z.h"
 
+/* XXX: Do not add the residual non-linearity model description   *
+ * to the final product until the model has been fully validated. *
+ * At the moment there are still discrepancies compared to the    *
+ * prototype implementation.                                      */
+#undef NONLINEARITY_MODEL
 
 typedef void (*muse_free_func)(void *);
 
@@ -731,10 +736,26 @@ muse_lingain_window_get_gain(const cpl_table *aWindowList,
         cpl_mask_or(mask, cpl_image_get_bpm(_flat2));
         cpl_image_reject_from_mask(_flat2, mask);
 
+        cpl_errorstate _state = cpl_errorstate_get();
+
         double median1 = cpl_image_get_median(_flat1);
         double median2 = cpl_image_get_median(_flat2);
-        double clip1   = aParams->sigma * cpl_image_get_stdev(_flat1);
-        double clip2   = aParams->sigma * cpl_image_get_stdev(_flat2);
+
+        if (!cpl_errorstate_is_equal(_state)) {
+          cpl_errorstate_set(_state);
+
+          cpl_array_set_invalid(_signal, iwindow);
+          cpl_array_set_invalid(_variance, iwindow);
+          cpl_array_set_invalid(_gain, iwindow);
+
+          cpl_image_delete(_flat2);
+          cpl_image_delete(_flat1);
+
+          continue;
+        }
+
+        double clip1 = aParams->sigma * cpl_image_get_stdev(_flat1);
+        double clip2 = aParams->sigma * cpl_image_get_stdev(_flat2);
 
         /* Remove outliers from the flat field pixel values by creating *
          * a bad pixel mask for these pixels. The resulting mask used   *
@@ -767,20 +788,31 @@ muse_lingain_window_get_gain(const cpl_table *aWindowList,
         cpl_image *dflat = cpl_image_subtract_create(_flat1, _flat2);
 
         if ((median1 < kMuseSaturationLimit) && (median2 < kMuseSaturationLimit)) {
+
+          _state = cpl_errorstate_get();
+
           double s1 = cpl_image_get_mean(_flat1);
           double s2 = cpl_image_get_mean(_flat2);
-          double s = 0.5 * (s1 + s2);
-          double v = 1.;
-          double g = 0.;
 
-          if ((cpl_array_get_double(aRon, iwindow, NULL) - mron_value) <= 5. * mron_sdev) {
-            double sdev = cpl_image_get_stdev(dflat);
-            v = 0.5 * sdev * sdev - mron_value * mron_value;
-            g = s / v;
+          if (!cpl_errorstate_is_equal(_state)) {
+            cpl_errorstate_set(_state);
+            cpl_array_set_invalid(_signal, iwindow);
+            cpl_array_set_invalid(_variance, iwindow);
+            cpl_array_set_invalid(_gain, iwindow);
+          } else {
+            double s = 0.5 * (s1 + s2);
+            double v = 1.;
+            double g = 0.;
+
+            if ((cpl_array_get_double(aRon, iwindow, NULL) - mron_value) <= 5. * mron_sdev) {
+              double sdev = cpl_image_get_stdev(dflat);
+              v = 0.5 * sdev * sdev - mron_value * mron_value;
+              g = s / v;
+            }
+            cpl_array_set_double(_signal, iwindow, s);
+            cpl_array_set_double(_variance, iwindow, v);
+            cpl_array_set_double(_gain, iwindow, g);
           }
-          cpl_array_set_double(_signal, iwindow, s);
-          cpl_array_set_double(_variance, iwindow, v);
-          cpl_array_set_double(_gain, iwindow, g);
         } else {
           cpl_array_set_invalid(_signal, iwindow);
           cpl_array_set_invalid(_variance, iwindow);
@@ -1828,6 +1860,7 @@ muse_lingain_compute(muse_processing *aProcessing,
     return -1;
   }
 
+#ifdef NONLINEARITY_MODEL
   /* Fit the residual detector non-linearity for each quadrant */
   cpl_table *linearity = muse_lingain_compute_nonlinearity(gaindata, windowlist,
                                                            gain, aParams);
@@ -1841,6 +1874,7 @@ muse_lingain_compute(muse_processing *aProcessing,
     muse_imagelist_delete(images);
     return -1;
   }
+#endif
 
   cpl_table_delete(gaindata);
   cpl_table_delete(windowlist);
@@ -1856,16 +1890,14 @@ muse_lingain_compute(muse_processing *aProcessing,
                                 "^DATASUM$|^DATAMIN$|^DATAMAX$|^DATAMD5$|"
                                 "^PCOUNT$|^GCOUNT$|^HDUVERS$|^BLANK$|"
                                 "^BZERO$|^BSCALE$|^BUNIT$|^CHECKSUM$|^INHERIT$|"
-                                "^PIPEFILE$|^ESO PRO ", 0);
+                                "^PIPEFILE$|^ESO PRO |" MUSE_WCS_KEYS, 0);
 
   const unsigned char nquadrant = 4;
   unsigned char iquadrant;
   for (iquadrant = 0; iquadrant < nquadrant; ++iquadrant) {
     char keyword[KEYWORD_LENGTH];
-    char comment[KEYWORD_LENGTH];
     unsigned char _iquadrant = iquadrant + 1;
-    int ncoefficient = (int)cpl_table_get_column_depth(linearity,
-                                                       "Coefficients");
+
     snprintf(keyword, KEYWORD_LENGTH, "ESO DET OUT%d GAIN", _iquadrant);
     cpl_propertylist_update_double(header, keyword,
                                    cpl_table_get_double(gain, "Gain",
@@ -1882,6 +1914,12 @@ muse_lingain_compute(muse_processing *aProcessing,
                                  "[ADU/e-] Conversion electrons to ADUs");
 #endif
 
+    snprintf(keyword, KEYWORD_LENGTH, QC_LINGAIN_GFITi_RMS, _iquadrant);
+    cpl_propertylist_append_double(header, keyword,
+                                   cpl_table_get_double(gain, "FitRms",
+                                                        iquadrant, NULL));
+
+#ifdef NONLINEARITY_MODEL
     snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_NONLINn_LLO, _iquadrant);
     cpl_propertylist_append_double(header, keyword,
                                    cpl_table_get_double(linearity, "SignalMin",
@@ -1896,6 +1934,9 @@ muse_lingain_compute(muse_processing *aProcessing,
     cpl_propertylist_set_comment(header, keyword,
                                  "[log10(ADU)] Maximum signal used for fit");
 
+    int ncoefficient = (int)cpl_table_get_column_depth(linearity,
+                                                       "Coefficients");
+
     snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_NONLINn_ORDER, _iquadrant);
     cpl_propertylist_append_double(header, keyword, ncoefficient - 1);
     cpl_propertylist_set_comment(header, keyword,
@@ -1905,6 +1946,7 @@ muse_lingain_compute(muse_processing *aProcessing,
         cpl_table_get_array(linearity, "Coefficients", iquadrant);
     int icoefficient;
     for (icoefficient = 0; icoefficient < ncoefficient; ++icoefficient) {
+      char comment[KEYWORD_LENGTH];
       double c = cpl_array_get_double(coefficients, icoefficient, NULL);
       snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_NONLINn_COEFFo,
                _iquadrant, (unsigned char)icoefficient);
@@ -1915,22 +1957,27 @@ muse_lingain_compute(muse_processing *aProcessing,
       cpl_propertylist_set_comment(header, keyword, comment);
     }
 
-    snprintf(keyword, KEYWORD_LENGTH, QC_LINGAIN_GFITi_RMS, _iquadrant);
-    cpl_propertylist_append_double(header, keyword,
-                                   cpl_table_get_double(gain, "FitRms",
-                                                        iquadrant, NULL));
     snprintf(keyword, KEYWORD_LENGTH, QC_LINGAIN_NLFITi_RMS, _iquadrant);
     cpl_propertylist_append_double(header, keyword,
                                    cpl_table_get_double(linearity, "FitRms",
                                                         iquadrant, NULL));
+#endif
   }
+
+#ifdef NONLINEARITY_MODEL
   cpl_table_erase_column(linearity, "FitRms");
 
   muse_processing_save_table(aProcessing, aParams->nifu, linearity, header,
                              MUSE_TAG_NONLINGAIN, MUSE_TABLE_TYPE_CPL);
+#else
+  muse_processing_save_header(aProcessing, aParams->nifu, header,
+                              MUSE_TAG_NONLINGAIN);
+#endif
 
   cpl_propertylist_delete(header);
+#ifdef NONLINEARITY_MODEL
   cpl_table_delete(linearity);
+#endif
   cpl_table_delete(gain);
 
   return 0;
