@@ -207,7 +207,14 @@ muse_lsf_params_compute(cpl_table *aLines, muse_pixtable *aPixtable,
 
   muse_lsf_params **lsfParams = cpl_calloc(n_slices + 1,
                                            sizeof(muse_lsf_params *));
-  #pragma omp parallel for default(none) num_threads(2)  /* as req. by Ralf */ \
+
+  // FIXME: Removed default(none) clause here, to make the code work with
+  //        gcc9 while keeping older versions of gcc working too without
+  //        resorting to conditional compilation. Having default(none) here
+  //        causes gcc9 to abort the build because of __func__ not being
+  //        specified in a data sharing clause. Adding a data sharing clause
+  //        makes older gcc versions choke.
+  #pragma omp parallel for num_threads(2)                                      \
           shared(aLines, aMaxIter, n_slices, i_slice, lsfParams, slice_pixtable)
   for (i_slice = 0; i_slice < n_slices; i_slice++) {
     cpl_errorstate es = cpl_errorstate_get();
