@@ -275,6 +275,25 @@ int main(int argc, char **argv)
            == CPL_ERROR_INCOMPATIBLE_INPUT); /* same error, even with FWHM */
   cpl_errorstate_set(es);
   cpl_propertylist_update_float(pt1->header, "EXPTIME", 1.);
+  /* missing header or negative value */
+  es = cpl_errorstate_get();
+  cpl_error_code rc = muse_xcombine_weights(pts, MUSE_XCOMBINE_HEADER);
+  cpl_test(!cpl_errorstate_is_equal(es) && rc == CPL_ERROR_NONE &&
+           cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND);
+  cpl_errorstate_set(es);
+  cpl_propertylist_update_float(pt1->header, "ESO DRS MUSE WEIGHT", -1.);
+  cpl_propertylist_update_float(pt2->header, "ESO DRS MUSE WEIGHT", 1.);
+  cpl_propertylist_update_float(pt3->header, "ESO DRS MUSE WEIGHT", -5.);
+  es = cpl_errorstate_get();
+  rc = muse_xcombine_weights(pts, MUSE_XCOMBINE_HEADER);
+  cpl_test(cpl_errorstate_is_equal(es) && rc == CPL_ERROR_NONE); /* just 2 warnings */
+  cpl_errorstate_set(es);
+  cpl_propertylist_erase(pt1->header, "ESO DRS MUSE WEIGHT");
+  cpl_propertylist_erase(pt2->header, "ESO DRS MUSE WEIGHT");
+  cpl_propertylist_erase(pt3->header, "ESO DRS MUSE WEIGHT");
+  cpl_table_erase_column(pt1->table, MUSE_PIXTABLE_WEIGHT); /* erase the ... */
+  cpl_table_erase_column(pt2->table, MUSE_PIXTABLE_WEIGHT); /* ... weights ... */
+  cpl_table_erase_column(pt3->table, MUSE_PIXTABLE_WEIGHT); /* ... again */
 
   /* test success cases of muse_xcombine_weights() */
   es = cpl_errorstate_get();
@@ -284,6 +303,24 @@ int main(int argc, char **argv)
   cpl_test(!cpl_table_has_column(pt2->table, MUSE_PIXTABLE_WEIGHT));
   cpl_test(!cpl_table_has_column(pt3->table, MUSE_PIXTABLE_WEIGHT));
   es = cpl_errorstate_get();
+  cpl_propertylist_update_float(pt1->header, "ESO DRS MUSE WEIGHT", 1.);
+  cpl_propertylist_update_float(pt2->header, "ESO DRS MUSE WEIGHT", 0.5);
+  cpl_propertylist_update_float(pt3->header, "ESO DRS MUSE WEIGHT", 2.);
+  cpl_test(muse_xcombine_weights(pts, MUSE_XCOMBINE_HEADER) == CPL_ERROR_NONE);
+  cpl_test(cpl_errorstate_is_equal(es));
+  cpl_test(cpl_table_has_column(pt1->table, MUSE_PIXTABLE_WEIGHT));
+  cpl_test(cpl_table_has_column(pt2->table, MUSE_PIXTABLE_WEIGHT));
+  cpl_test(cpl_table_has_column(pt3->table, MUSE_PIXTABLE_WEIGHT));
+  cpl_test(cpl_table_get_float(pt1->table, MUSE_PIXTABLE_WEIGHT, 0, NULL) == 1.);
+  cpl_test(cpl_table_get_float(pt2->table, MUSE_PIXTABLE_WEIGHT, 0, NULL) == 0.5 * 2.);
+  cpl_test(cpl_table_get_float(pt3->table, MUSE_PIXTABLE_WEIGHT, 0, NULL) == 2. * 3.);
+  cpl_table_erase_column(pt1->table, MUSE_PIXTABLE_WEIGHT); /* erase the ... */
+  cpl_table_erase_column(pt2->table, MUSE_PIXTABLE_WEIGHT); /* ... weights ... */
+  cpl_table_erase_column(pt3->table, MUSE_PIXTABLE_WEIGHT); /* ... again */
+  cpl_propertylist_erase(pt1->header, "ESO DRS MUSE WEIGHT");
+  cpl_propertylist_erase(pt2->header, "ESO DRS MUSE WEIGHT");
+  cpl_propertylist_erase(pt3->header, "ESO DRS MUSE WEIGHT");
+  es = cpl_errorstate_get();
   cpl_test(muse_xcombine_weights(pts, MUSE_XCOMBINE_EXPTIME) == CPL_ERROR_NONE);
   cpl_test(cpl_errorstate_is_equal(es));
   cpl_test(cpl_table_has_column(pt1->table, MUSE_PIXTABLE_WEIGHT));
