@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2014-2015 European Southern Observatory
+ * Copyright (C) 2014-2016 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,8 +66,9 @@ int main(int argc, char **argv)
 {
   const char *idstring = "muse_cube_combine";
   cpl_init(CPL_INIT_DEFAULT);
-  cpl_msg_set_level(CPL_MSG_DEBUG);
   cpl_msg_set_time_on();
+  muse_processing_recipeinfo(NULL);
+  cpl_msg_set_level(CPL_MSG_DEBUG);
   cpl_msg_set_component_on();
   cpl_errorstate state = cpl_errorstate_get();
 
@@ -133,6 +134,7 @@ int main(int argc, char **argv)
   double crpix1ref = NAN, crpix2ref = NAN, crval1ref = NAN, crval2ref = NAN,
          cd11ref = NAN, cd12ref = NAN, cd21ref = NAN, cd22ref = NAN;
   char *ctype3ref = NULL;
+  cpl_boolean loglambda = CPL_FALSE;
   double crpix3ref = NAN,
          crval3ref = NAN,
          cd33ref = NAN;
@@ -232,6 +234,8 @@ int main(int argc, char **argv)
       crpix3ref = crpix3;
       crval3ref = crval3;
       cd33ref = cd33;
+      loglambda = ctype3ref && (!strncmp(ctype3ref, "AWAV-LOG", 9) ||
+                                !strncmp(ctype3ref, "WAVE-LOG", 9));
     } else if (strncmp(ctype3ref, ctype3, strlen(ctype3ref)) ||
                fabs(crpix3 - crpix3ref) > DBL_EPSILON ||
                fabs(crval3 - crval3ref) > DBL_EPSILON ||
@@ -243,6 +247,10 @@ int main(int argc, char **argv)
     }
     double lpx1 = (1. - crpix3) * cd33 + crval3,
            lpx2 = (naxis3 - crpix3) * cd33 + crval3;
+    if (loglambda) {
+      lpx1 = crval3 * exp((1 - crpix3) * cd33 / crval3);
+      lpx2 = crval3 * exp((naxis3 - crpix3) * cd33 / crval3);
+    }
     cpl_msg_debug(idstring, "Cube %d, axis 3 WCS: %f %f %f (%s, %d pixels, "
                   "%f..%f %s)", ic + 1, crpix3, crval3, cd33, ctype3, naxis3,
                   lpx1, lpx2, cunit3);
@@ -302,6 +310,10 @@ int main(int argc, char **argv)
     /* compute good valid range of pixels in dispersion direction */
     int l1 = lround((lmin - crval3) / cd33 + crpix3),
         l2 = lround((lmax - crval3) / cd33 + crpix3);
+    if (loglambda) {
+      l1 = lround(crval3 / cd33 * log(lmin / crval3)) + crpix3;
+      l2 = lround(crval3 / cd33 * log(lmax / crval3)) + crpix3;
+    }
     cpl_msg_debug(idstring, "Cube %d: %d..%d (%f..%f Angstrom)", ic + 1,
                   l1, l2, lmin, lmax);
     if (l1 <= 1) {
@@ -433,10 +445,9 @@ int main(int argc, char **argv)
   cpl_msg_debug(idstring, "%d of %d planes left in first cube",
                 (int)cpl_imagelist_get_size(cube->data), nplanes);
   nplanes = cpl_imagelist_get_size(cube->data);
-  /* update the spectral part of the WCS with the new setup */
-  cpl_propertylist_update_double(cube->header, "CRPIX3", 1.);
-  cpl_propertylist_update_double(cube->header, "CRVAL3",
-                                 cpl_table_get(table, "LMIN", 0, NULL));
+  /* update the spectral part of the WCS with the new setup, i.e. *
+   * adjust CRPIX3 by the number of deleted planes at the start   */
+  cpl_propertylist_update_double(cube->header, "CRPIX3", crpix3ref - (l1 - 1));
 
   /* one-by-one fully load the other cubes, and copy *
    * the valid data from them into the output cube   */
