@@ -153,7 +153,7 @@ int main(int argc, char **argv)
   if (!files) {
     files = soffiles;
   } else {
-    if (cpl_array_get_size(soffiles) > 0) {
+    if (soffiles && cpl_array_get_size(soffiles) > 0) {
       cpl_array_insert(files, soffiles, cpl_array_get_size(files));
     }
     cpl_array_delete(soffiles);
